/* MOLDFLOW COPYRIGHT: (c) Copyright 2004 Moldflow Corporation.  All rights reserved. */
//------------------------------------------------------------------------------

#include <stdlib.h>
#include <stdio.h>
#include <vector>

//------------------------------------------------------------------------------

// libftp - GNU Library General Public License
#include "libftp.h"

//------------------------------------------------------------------------------

#pragma message ( "Automatically link WINSOCK" )
#pragma comment ( lib, "wsock32.lib" )

//------------------------------------------------------------------------------

std::vector<void*> gHeap ;

//------------------------------------------------------------------------------

static void FtpExit ()
{
	for ( std::vector<void*>::iterator i = gHeap . begin () ; i != gHeap . end () ; ++ i )
	{
		if ( *(i) != NULL )
		{
			free ( *(i) ) ;
			*(i) = NULL ;
		}
	}
	gHeap . clear () ;
}

//------------------------------------------------------------------------------

void* FtpAlloc ( size_t aSize )
{
	static int lHeapExit = atexit ( FtpExit ) ;

	void* lMem = malloc ( aSize ) ;
	if ( lMem ) 
	{
		memset ( lMem, 0, aSize ) ;
		gHeap . push_back ( lMem ) ;
	}
	return lMem ;
}

//------------------------------------------------------------------------------

void FtpFree ( void* lMem )
{
	if ( lMem )
	{
		for ( std::vector<void*>::iterator i = gHeap . begin () ; i != gHeap . end () ; ++ i )
		{
			if ( *(i) != NULL && *(i) == lMem )
			{
				free ( *(i) ) ;
				*(i) = NULL ;
				return ;
			}
		}
	}
}

//------------------------------------------------------------------------------

static int FtpTimeOut ( netbuf* , int , void* )
{
	// If this function gets called an error has occurred and ftp protocol has timed out.
	return 0 ;
}

//------------------------------------------------------------------------------
int FtpGetFile ( const char* aHost, const char* aUser, const char* aPassword, const char* aRemoteFile, const char* aLocalFile )
{
	int    lResult = 0 ;
	netbuf* lConnection = NULL ;
	
	// Init ftp library
	FtpInit () ;

	// Connect to host
	if ( FtpConnect ( aHost, &lConnection ) )
	{
		// Login to host
		if ( FtpLogin ( aUser, aPassword, lConnection ) )
		{
			// Set connection status
			FtpOptions ( FTPLIB_CONNMODE, FTPLIB_PASSIVE, lConnection ) ; 

			// Set idle timeout to 60000 milli seconds i.e. 1 minute
			FtpOptions ( FTPLIB_IDLETIME, 60000, lConnection ) ;

			// Set callback function to  time out
			FtpOptions ( FTPLIB_CALLBACK, (size_t) FtpTimeOut, lConnection ) ;

			// Just incase zap local file
			remove ( aLocalFile ) ;

			// Get file
			if ( FtpGet ( aLocalFile, aRemoteFile, FTPLIB_IMAGE, lConnection ) )
			{
				lResult = 1 ;	
			}
		}	
	}

	// Close connections
	if ( lConnection )
	{
		FtpClose ( lConnection ) ;	
	}

	return lResult ;	
}

//------------------------------------------------------------------------------

int FtpPutFile ( const char* aHost, const char* aUser, const char* aPassword, const char* aRemoteFile, const char* aLocalFile )
{
	int    lResult = 0 ;
	netbuf* lConnection = NULL ;
	
	// Init ftp library
	FtpInit () ;

	// Connect to host
	if ( FtpConnect ( aHost, &lConnection ) )
	{
		// Login to host
		if ( FtpLogin ( aUser, aPassword, lConnection ) )
		{
			// Set connection status
			FtpOptions ( FTPLIB_CONNMODE, FTPLIB_PASSIVE, lConnection ) ; 

			// Set idle timeout to 60000 milli seconds i.e. 1 minute
			FtpOptions ( FTPLIB_IDLETIME, 60000, lConnection ) ;

			// Set callback function to  time out
			FtpOptions ( FTPLIB_CALLBACK, (size_t) FtpTimeOut, lConnection ) ;

			// Remove existing file if we can
			FtpDelete ( aRemoteFile, lConnection ) ;

			// Put file
			if ( FtpPut ( aLocalFile, aRemoteFile, FTPLIB_IMAGE, lConnection ) )
			{
				lResult = 1 ;	
			}
		}	
	}

	// Close connections
	if ( lConnection )
	{
		FtpClose ( lConnection ) ;	
	}

	return lResult ;	
}

//------------------------------------------------------------------------------

